#version 430 core
/*
* Copyright 2010 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;

out vec4 color;

uniform vec3 camera_world;

uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular 
uniform float specularAlpha;
uniform float ambientBrightness;
uniform sampler2D diffuseTexture;
uniform bool hudRendering;

uniform bool noiseRendering;
uniform vec4 DarkWoodColor = vec4( 0.8, 0.5, 0.1, 1.0 );
uniform vec4 LightWoodColor = vec4( 1.0, 0.75, 0.25, 1.0 );
uniform mat4 Slice;



uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirL;

uniform struct PointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL;


vec3 phong(vec3 n, vec3 l, vec3 v, vec3 diffuseC, float diffuseF, vec3 specularC, float specularF, float alpha, bool attenuate, vec3 attenuation) {
	float d = length(l);
	l = normalize(l);
	float att = 1.0;	
	if(attenuate) att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	vec3 r = reflect(-l, n);
	return (diffuseF * diffuseC * max(0, dot(n, l)) + specularF * specularC * pow(max(0, dot(r, v)), alpha)) * att; 
}

void main() {	
	vec3 n = normalize(vert.normal_world);
	vec3 v = normalize(camera_world - vert.position_world);
	

	vec4 texColor;
	if (noiseRendering == true) {
	vec4 cyl = Slice * vec4( vert.uv.st, 0.0, 1.0 );

 	// The distance from the log's y axis.
 	float dist = length(cyl.xz);

 	// Perturb the distance using the noise texture
 	vec4 noise = texture(diffuseTexture, vert.uv);

 	dist += noise.b;

 	// Determine the color as a mixture of the light and
 	// dark wood colors.
	//float t = abs( fract(14.46f) * 2.0f);
 	float t = 1.0f - abs( fract( dist ) * 2.0f - 1.0f );
 	//t = smoothstep( 0.2, 0.5, t );
 	vec4 color = mix( DarkWoodColor, LightWoodColor, t );
 	texColor = vec4( color.rgb , 1.0 );
	} else {
	texColor = texture(diffuseTexture, vert.uv).rgba;
	}

color = vec4(texColor.rgb * (materialCoefficients.x * ambientBrightness), texColor.a); // ambient
	
if (hudRendering == false){	
	// add directional light contribution
	color.rgb += phong(n, -dirL.direction, v, dirL.color * texColor.rgb, materialCoefficients.y, dirL.color, materialCoefficients.z, specularAlpha, false, vec3(0));
			
	// add point light contribution
	//color.rgb += phong(n, pointL.position - //vert.position_world, v, pointL.color * texColor.rgb, //materialCoefficients.y, pointL.color, //materialCoefficients.z, specularAlpha, true, //pointL.attenuation);
}
color = color;
	
}


